/***********************************************************************************************************************
* Copyright (C) All rights reserved.
***********************************************************************************************************************/

/***********************************************************************************************************************
* @file    LIN.c
* @brief   This file implements device driver for LIN module.
* @version 1.0.0  
* @date    2022/2/25
***********************************************************************************************************************/


/***********************************************************************************************************************
Includes
***********************************************************************************************************************/

#include "userdefine.h"
#include "BAT32G113.h"
#include "lin.h"


uint8_t Master_TxData1[]={0x08,0x80,0x20};   /*Transmission data store array*/
uint8_t Master_TxData2[]={0x49,0x40,0x10};   /*Transmission data store array*/
uint8_t Master_TxData3[]={0xCA,0x0F,0x0F};   /*Transmission data store array*/

uint8_t Master_RxData1[8];
uint8_t Master_RxData2[8];
uint8_t Master_RxData3[8];


#ifdef LIN_MASTER

/***********************************************************************************************************************
* Function Name: LIN_Master_Init(void)
* Description  : This function initializes the RLIN Slave node, setting clock supply,baud rate,ect.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/

void LIN_Master_Init(void)
{
  CGC->PER2 |= CGC_PER2_LINEN_Msk;          /*  Enable input clock supply LIN*/
  CGC->LINCKSEL=0x00;        /*  selects the  fclk=32MHz  clock to LIN.*/
  LIN->LWBR  = 0x01;         /*  b0=1, LIN2.0 or 2.1;  Prescaler Clock Selcet 1/1;  bit sampling count select 0000 : 16 sampling. */ 
  LIN->LBRP0 = 0x67;         /*  fa:0X67=103D, Baud rate= 32M/ (103+1)*16= 19230 bps  fb:9615bps    fc:2403bps */
  LIN->LBRP1 = 0x5F;         /*  fd:10416bps*/
  
  INTC_ClearPendingIRQ(LINR_IRQn);
  INTC_ClearPendingIRQ(LINT_IRQn);
  INTC_ClearPendingIRQ(LINS_IRQn);
  INTC_EnableIRQ(LINR_IRQn);
  INTC_EnableIRQ(LINT_IRQn);
  INTC_EnableIRQ(LINS_IRQn);

  LIN->LIE  |=  0x07;        /*  Enable successful response/wake-up reception interrupt, enable all interrupt*/
  LIN->LEDE |=  0x8F;        /*  Enable error detection */
  
   /*  Header format setting*/
  LIN->LMD  =   0x10;         /*  b0b1=00: LIN master mode ; b3b2=00: fa=LIN sysclock;  b4=1:transmission interrupt,sucessful reception interrupt...; b5=0: The noise filter is enable.*/
  LIN->LBFC =   0x15;         /*  b3-b0=0101: transmission break width 18Tbits; b5b4=01: break delimiter 2Tbit*/
  LIN->LSC  =   0x11;         /*  b2-b0=001:inter-byte space 1bit or Response space 4bit; b5b4=01: inter-byte space 1Tbit;*/
  LIN->LWUP =   0x30;         /*  b7-b4=0100: Wake-up Transmission low width 4 bits.*/
  LIN->LIDB &=  0x00;         /*  Clear the ID buffer */
 
  MISC->ISC   =   0x00;        /*  INTP11 pin input signal is set as external interrupt input*/
  CGC->LINCKSEL|=0x10;         /*  Enable LIN engine clock supply,*/
  LTXD0_PORT_SETTING();
  LRXD0_PORT_SETTING();
}

/***********************************************************************************************************************
* Function Name: LIN_Master_HeaderTransmit(uint8_t ID)
* Description  : This function is setting in slave mode, enable header reception is started.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void LIN_Master_HeaderTransmit(uint8_t ID)
{
 
  LIN->LCUC = 0x03;              /*  01: RLIN rest mode is canceled; 03:RLIN operation mode */
  LIN->LIDB = 0X00;             /*  clear ID buffer */
  LIN->LIDB = ID;              /*  ID  load to ID buffer */ 
  Clear_DataBuffer();     /* clear all data buffer */
  
  switch(ID)                
      {
        case 0x08: LIN_Master_Transmit(Master_TxData1,3);       /* ready for response transmit*/
                    break;
        case 0x49: LIN_Master_Transmit(Master_TxData2,3);       /* ready for response transmit*/
                    break;
        case 0xCA: LIN_Master_Transmit(Master_TxData3,3);       /* ready for response transmit*/
                    break;
        case 0x8B: LIN_Master_Receive(2);                       /* ready for response receive*/
                    break;
        default:   break;
        }
    
    LIN->LTRC=0x01;  /*FTS=1,Frame transmit is started*/
}

/***********************************************************************************************************************
* Function Name: LIN_Master_Transmit(void)
* Description  : This function seting data buffer for response transmission start
* Arguments    : uint8_t* databuf    : variable array data. 
                 uint8_t Data_length : transmit data length.
* Return Value : None
***********************************************************************************************************************/
void LIN_Master_Transmit(uint8_t * databuf,uint8_t Data_length)
{
  uint8_t i;
  volatile uint8_t * Databuf_adr;
  LIN->LDFC=0x30;
  LIN->LDFC|=Data_length;    /*  MSB=0011, RCDS=1:Transmission, LCS=1: Enhanced checksum mode;  LSB=0100: response data lengh select 4 byte*/
  Databuf_adr=&LIN->LDB1;
  for(i=0;i<Data_length;i++)
  {
     *((uint8_t *)(Databuf_adr+i))=databuf[i];  
  }   
}

/***********************************************************************************************************************
* Function Name: LIN_Master_Receive(void)
* Description  : This function clear data buffer ready for response reception start
* Arguments    : uint8_t Data_length : receive data length.
* Return Value : None
***********************************************************************************************************************/
void LIN_Master_Receive(uint8_t Data_length)
{

  Clear_DataBuffer();   
  LIN->LDFC=0x20;
  LIN->LDFC|=Data_length;   /* MSB=0011, RCDS=1:Transmission, LCS=1: Enhanced checksum mode;  LSB=0100: response data lengh select 4 byte*/
}


/***********************************************************************************************************************
* Function Name: LIN_Master_GetData(void)
* Description  : This function is get data from data buffr according to ID data during the response reception
* Arguments    : uint8_t Data_length : receive data length.
* Return Value : None
***********************************************************************************************************************/

void LIN_Master_GetData(void)
{
    switch(LIN->LIDB)
    {
    case 0x8B: Get_response_RxData(Master_RxData1);           
             break;
  /*  case 0x4c: Get_reponse_RxData(Master_RxData2);      
             break;*/
 /*    case 0x0D: Get_reponse_RxData(Master_RxData3);      
              break,*/            
    default: break;    
      }
}

   
#endif


#ifdef LIN_SLAVE

/***********************************************************************************************************************
* Function Name: LIN_Slave_Init(void)
* Description  : This function initializes the RLIN Slave node, setting clock supply,baud rate,ect.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/

void LIN_Slave_Init(void)
{
  CGC->PER2 |= CGC_PER2_LINEN_Msk;         /*  Enable input clock supply LIN. */
  CGC->LINCKSEL=0x00;      /*  selects the  fclk/2  clock to LIN.*/
  LIN->LWBR =  0x34;      /*  b3-b1=010: Prescaler Clock Selcet 32/4, bit sampling count select b7-b4=0011 : 4 sampling. */ 
  LIN->LBRP0 = 0x67;      /*  lower 8bit : 0X67=103D, Baud rate= 32M/ (103+1)*16= 19230 bps*/
  LIN->LBRP1 = 0x00;      /*  upper 8 bits in the 16bit counter of the baud rate prescaler*/
  
  INTC_ClearPendingIRQ(LINR_IRQn);
  INTC_ClearPendingIRQ(LINT_IRQn);
  INTC_ClearPendingIRQ(LINS_IRQn);
  INTC_EnableIRQ(LINR_IRQn);
  INTC_EnableIRQ(LINT_IRQn);
  INTC_EnableIRQ(LINS_IRQn);

  LIN->LIE  |=  0x0F;      /*  Enable successful response/wake-up reception interrupt, enable all interrupt*/
  LIN->LEDE |=  0xC9;      /*  Enable error detection */
  
  /*Header format setting*/
  LIN->LMD  =  0x12;        /*  b1b0=10; LIN Slave mode (Auto baud rate),transmission interrupt,sucessful reception interrupt..., The noise filter is enable.*/
  LIN->LBFC =  0x00;        /*  Reception break of 9.5/10 or more Tbits*/
  LIN->LSC  =  0x24;        /*  Response space 4bit; inter-byte space 1bit;*/
  LIN->LWUP =  0x30;        /*  Wake-up Transmission low width 4 bits.*/
  LIN->LIDB&=  0x00;        /*  Clear the ID buffer */
 
  MISC->ISC   =  0x04;         /*  LRXD0 pin input signal is set as external interrupt input,*/
  CGC->LINCKSEL|=0x10;        /*  Enable LIN engine clock supply,*/
  
  LTXD0_PORT_SETTING();
  LRXD0_PORT_SETTING();
}

/***********************************************************************************************************************
* Function Name: LIN_Slave_HeaderReceive(void)
* Description  : This function is setting in slave mode, enable header reception is started.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void LIN_Slave_HeaderReceive(void)
{
  
  LIN->LCUC = 0x03;          /* 01: RLIN rest mode is canceled; 03:RLIN operation mode  */
  LIN->LTRC|= 0x01;          /* FTS=1; Header reception or wake up transmission/reception is started.*/
}


/***********************************************************************************************************************
* Function Name: LIN_Slave_Transmit(void)
* Description  : This function seting data buffer for response transmission start
* Arguments    : uint8_t* databuf    : variable array data. 
                 uint8_t Data_length : transmit data length.
* Return Value : None
***********************************************************************************************************************/

void LIN_Slave_Transmit(uint8_t* databuf,uint8_t Data_length)
{ 
  uint8_t i;
  volatile uint8_t * Databuf_adr;   
  LIN->LDFC=0x30;                    /*b5=1:enhanced checksum mode; b4=1:transmission*/
  LIN->LDFC|=Data_length;            /* b4-b0=Data_length: response data lengh select byte*/
  Databuf_adr=&LIN->LDB1;   /* get the data buffer address*/
  for(i=0;i<Data_length;i++)             /* setting tansmission data to date buffer*/
  {
    *((uint8_t *)(Databuf_adr+i))=databuf[i];  
  }    
  LIN->LTRC=0x02;                     /*setting RTS=1;Response transmission start*/   
}

/***********************************************************************************************************************
* Function Name: LIN_Slave_Receive(void)
* Description  : This function clear data buffer for response reception start
* Arguments    : uint8_t Data_length : receive data length.
* Return Value : None
***********************************************************************************************************************/

void LIN_Slave_Receive(uint8_t Data_length)
{
    Clear_DataBuffer();
    LIN->LDFC=0x20;                   /*b5=1:enhanced checksum mode; b4=0:Reception*/
    LIN->LDFC|=Data_length;           /* b4-b0=Data_length: response data lengh select byte*/
    LIN->LTRC=0x02;                   /*setting RTS=1,response reception is started*/
}

void LIN_Slave_NoResponse(void) 
{
    LIN->LTRC=0x04;                     /* setting LNRR=0, No response request*/
}

#endif

/***********************************************************************************************************************
* Function Name: Clear_DataBuffer
* Description  : This function setting all data buffer to some value
* Arguments    : uint8_t x : setting data buff value
* Return Value : None
***********************************************************************************************************************/

void Clear_DataBuffer()
{   
  uint8_t i;
  volatile uint8_t * Databuf_adr;
  Databuf_adr=&LIN->LDB1;
  for(i=0;i<8;i++)
  {
   *((uint8_t *)(Databuf_adr+i))=0U;
  } 

}

/***********************************************************************************************************************
* Function Name: Get_response_RxData
* Description  : This function get data buffer value to a variable array  
* Arguments    : uint8_t * RxData : a avriable array for store Data
* Return Value : None
***********************************************************************************************************************/
uint8_t  Get_response_RxData(uint8_t * RxData)
{
  uint8_t i,k;
  volatile uint8_t * Databuf_adr;
  k=LIN->LDFC&0x0F;
  Databuf_adr=&LIN->LDB1;
  for(i=0;i<k;i++)
  {
    RxData[i]=(*((uint8_t *)(Databuf_adr+i)));  
  } 
  return RxData[1];
}

void LIN_unInit(void){
  CGC->PER2 &= ~CGC_PER2_LINEN_Msk;          /*  Disable input clock supply LIN*/
}


